//==============================================================================
//
//    lossyWAV: Added noise WAV bit reduction method by David Robinson;
//              Noise shaping coefficients by Sebastian Gesemann;
//
//    Copyright (C) 2007, 2008 Nick Currie, Copyleft.
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//    Contact: lossywav <at> yahoo <dot> com
//
//==============================================================================
unit ntypes;
//==============================================================================
{$DEFINE CPU386}
{$N+ $A8}
//==============================================================================
interface
//==============================================================================
const
  Array_Max  = 131071;
  crlf = #13#10;

  Single_Size     = 4;
  Single_Shift    = 2;

  Double_Size     = 8;
  Double_Shift    = 3;

  Integer_Size  = 4;
  Integer_Shift = 2;

  Int64_Size    = 8;
  Int64_Shift   = 3;

  DoubleComplex_Size  = 16;
  DoubleComplex_Shift = 4;

  Pointer_Size  = 4;
  Pointer_Shift = 2;

  TwoPi      = 6.28318530717958647693;  { 2*Pi          }
  HalfPi     = 1.57079632679489661923;  { Pi/2          }
  Log2x20    = 6.02059991327962390427;  { 20 * log10(2) }
  MaxDb      = 5623413251903.49080395;  { 10^(255/20)   }

  min_sample = -2147483648;
  max_sample =  2147483647;

type
  STimer_Record = Packed Record
    Start     : Int64;
    Stop      : Int64;
    Frequency : Int64;
    Elapsed   : Double;
  End;

  TComplex = Packed Record
                Re, Im : Double;
  End;

  TShaping = Packed Record
                A, B : array[0..7] of Double;
  end;

  TwoPowerComplexRecord = packed record
    p00 : array[0..    1] of TComplex; // 2^00
    p01 : array[0..    1] of TComplex; // 2^01
    p02 : array[0..    3] of TComplex; // 2^02
    p03 : array[0..    7] of TComplex; // 2^03
    p04 : array[0..   15] of TComplex; // 2^04
    p05 : array[0..   31] of TComplex; // 2^05
    p06 : array[0..   63] of TComplex; // 2^06
    p07 : array[0..  127] of TComplex; // 2^07
    p08 : array[0..  255] of TComplex; // 2^08
    p09 : array[0..  511] of TComplex; // 2^09
    p0A : array[0.. 1023] of TComplex; // 2^10
    p0B : array[0.. 2047] of TComplex; // 2^11
    p0C : array[0.. 4095] of TComplex; // 2^12
    p0D : array[0.. 8191] of TComplex; // 2^13
  end;

  TwoPowerDoubleRecord = packed record
    p00 : array[0..    1] of Double; // 2^00
    p01 : array[0..    1] of Double; // 2^01
    p02 : array[0..    3] of Double; // 2^02
    p03 : array[0..    7] of Double; // 2^03
    p04 : array[0..   15] of Double; // 2^04
    p05 : array[0..   31] of Double; // 2^05
    p06 : array[0..   63] of Double; // 2^06
    p07 : array[0..  127] of Double; // 2^07
    p08 : array[0..  255] of Double; // 2^08
    p09 : array[0..  511] of Double; // 2^09
    p0A : array[0.. 1023] of Double; // 2^10
    p0B : array[0.. 2047] of Double; // 2^11
    p0C : array[0.. 4095] of Double; // 2^12
    p0D : array[0.. 8191] of Double; // 2^13
  end;

  TwoPowerIntegerRecord = packed record
    p00 : array[0..    1] of Integer; // 2^00
    p01 : array[0..    1] of Integer; // 2^01
    p02 : array[0..    3] of Integer; // 2^02
    p03 : array[0..    7] of Integer; // 2^03
    p04 : array[0..   15] of Integer; // 2^04
    p05 : array[0..   31] of Integer; // 2^05
    p06 : array[0..   63] of Integer; // 2^06
    p07 : array[0..  127] of Integer; // 2^07
    p08 : array[0..  255] of Integer; // 2^08
    p09 : array[0..  511] of Integer; // 2^09
    p0A : array[0.. 1023] of Integer; // 2^10
    p0B : array[0.. 2047] of Integer; // 2^11
    p0C : array[0.. 4095] of Integer; // 2^12
    p0D : array[0.. 8191] of Integer; // 2^13
  End;

  TypeDoubleVector  = array[0..Array_Max] of Double;
  TypeIntegerVector = array[0..Array_Max] of Integer;
  TypeComplexVector = array[0..Array_Max] of TComplex;

  PtrDoubleVector  = ^TypeDoubleVector;
  PtrIntegerVector = ^TypeIntegerVector;
  PtrComplexVector = ^TypeComplexVector;

const
  complex_zero : tcomplex = (Re:0;Im:0);

//==============================================================================
// lookup tables.
//==============================================================================
var
  PowersOfTen     : array[0..15] of Int64;
  PowersOfTwo     : array[0..63] of Int64;
  PowersOfTwoM1   : array[0..63] of Int64;
  PowersOfTwoInv  : array[0..63] of Double;
  OneOverX        : array[0..16385] of Double;

//==============================================================================
// standard output files.
//==============================================================================
  ToLogFile : Text;

  hSTDIN    : tHandle;
  hSTDOUT   : tHandle;
  hSTDERR   : tHandle;

  ftSTDIN   : Cardinal;
  ftSTDOUT  : Cardinal;
  ftSTDERR  : Cardinal;

  procedure ndelay(dt : double);

//==============================================================================
implementation
//==============================================================================
// Unit ntypes end;
//==============================================================================
uses
//==============================================================================
  windows,
  nmaths;
//==============================================================================
var
//==============================================================================
// working loop counter.
//==============================================================================
  nt_i, nt_j : Integer;
  ntypes_packer : array[0..3] of integer;
//==============================================================================
  procedure ndelay(dt : double);
  var
    stimer : stimer_record;
  begin
    QueryPerformanceFrequency(STimer.Frequency);
    QueryPerformanceCounter(STimer.Start);
    repeat
      QueryPerformanceCounter(STimer.Stop);
      asm
        nop; nop; nop; nop;
        nop; nop; nop; nop;
        nop; nop; nop; nop;
        nop; nop; nop; nop;
      end;
    until (Stimer.Stop-STimer.Start)>Stimer.Frequency*dt;
  end;


//==============================================================================
initialization
//==============================================================================
  ntypes_packer[0]:=0;

  for nt_j:=0 to 62 do
  begin
    nt_i:=nt_j;
    powersoftwo[nt_j]:=twototheposintpower(nt_i);
    powersoftwom1[nt_j]:=powersoftwo[nt_j]-1;
    powersoftwoinv[nt_j]:=1/powersoftwo[nt_j];
  end;

  powersoften[0]:=1;
  for nt_j:=1 to 15 do
    powersoften[nt_j]:=powersoften[nt_j-1]*10;

  OneOverX[0]:=1;
  for nt_j:=1 to 16385 do
  asm
                fld1
                fidiv     nt_j
                mov       eax,nt_j
                fstp      qword ptr OneOverX[eax*double_size]
  end;
//==============================================================================
end.
//==============================================================================

